class Priority:
    """
    Priority of topic

    Attributes
    ----------
    Highest : int
        Highest (100)
    High : int
        High (75)
    Normal : int
        Normal (50)
    Low : int
        Low (25)
    Ignore : int
        Ignore (0)
    """
    Highest = 100
    High = 75
    Normal = 50
    Low = 25
    Ignore = 0
